import 'dart:math';

import 'package:flutter/material.dart';

class ToggleRotate extends StatefulWidget {

  final double beginAngle;
  final double endAngle;
  final Widget child;
  final Curve curve;
  final int durationMs;
  final bool isRotate;


  ToggleRotate({
    required this.child,
    this.beginAngle = 0.0,
    this.endAngle = 0.0,
    this.curve = Curves.fastOutSlowIn,
    this.durationMs = 200,
    this.isRotate = false,
  });

  @override
  _ToggleRotateState createState() => _ToggleRotateState();
}

class _ToggleRotateState extends State<ToggleRotate> with SingleTickerProviderStateMixin{

  late AnimationController _controller;
  late Animation<double> _turns;
  late bool _red;


  @override
  void initState() {
    super.initState();
    _controller = AnimationController(vsync: this, duration: Duration(milliseconds: widget.durationMs));
    _turns = Tween<double>(
        begin: widget.beginAngle / 180 * pi,
        end: widget.endAngle / 180 * pi
    ).animate(_controller);
    _red = widget.isRotate;
  }


  @override
  void dispose() {
    super.dispose();
    _controller.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if(_red != widget.isRotate){
      if(widget.isRotate){
        _controller.forward();
      }else{
        _controller.reverse();
      }
      _red = widget.isRotate;
    }

    return AnimatedBuilder(
      animation: _controller,
      builder: (BuildContext context, Widget? child) => Transform(
        transform: Matrix4.rotationZ(_turns.value),
        alignment: Alignment.center,
        child: widget.child,
      ),
    );
  }
}
